/*
 * Kerberos 5-2-4 remapping code
 *
 * This code implements a special principal "remapping" function.
 * This motivation for this is as follows:
 *
 * When doing cross-realm authentication with AFS, the PTS id for
 * foreign realm users ends up not matching the Unix userid.  This
 * subtly breaks a number of things.
 *
 * This remapping changes a V4 principal from a principal in a foreign
 * realm to one in a local realm, based on a set of mapping criteria.
 * Note that we only do this when getting service tickets for AFS.
 *
 * The security implications here are nasty; be careful.
 */

#include <afsconfig.h>
#include <afs/param.h>
#if defined(HAVE_KRB5_CREDS_KEYBLOCK) || defined(HAVE_KRB5_CREDS_SESSION)
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/signal.h>
#include <netinet/in.h>
#ifndef DEFAULT_PROFILE_PATH
#ifdef AFS_DARWIN_ENV
#define DEFAULT_PROFILE_PATH "/Library/Preferences/edu.mit.Kerberos"
#else
#define DEFAULT_PROFILE_PATH "/etc/krb5.conf"
#endif
#endif
#include <rx/rxkad.h>

static int remap_initialized = 0;
static void *remap_cf;

/*
 * Initialize the remapping library
 */

int
init_remap(char *file)
{
    int code;
    if (remap_initialized)
	return 0;

    /*
     * Because I'm lazy, I decided to use the already-existing
     * Kerberos profile routines, even though it's perhaps not the
     * best file format.
     */
    if ( !file )
	file = DEFAULT_PROFILE_PATH;

    code = afs_config_parse_file(file, &remap_cf);

    if (code)
	return code;

    remap_initialized = 1;
    return 0;
}

int
remap_principal(char *part0, char *part1, char *part2, char *realmp,
		char *name, char *inst)
{
    char *inprinc;
    char *oldinst, *newinst;
    int slen;

    if (! remap_initialized)
	return 1;

    strncpy(name, part0, MAXKTCNAMELEN);
    name[MAXKTCNAMELEN - 1] = '\0';

    /*
     * First off, check to see if we can remap this principal
     * via the user-specific rules.  Note that this is called
     * only when the service is afs!
     *
     * Supported rule forms are
     *	[instancemapping]
     *	    afs = {
     *		oldinstance = newinstance
     *		oldinstance/* = newinstance
     *	    }
     *	[usermapping]
     *	    afs = {
     *		princ1@REALM1 = princ2@REALM2
     *	    }
     */

    /* if there are any quoted /, @, or \ characters, we won't play */
    if ( strchr(part0, '\\') )
	return 1;
    if ( strchr(part1, '\\') )
	return 1;
    if ( strchr(part2, '\\') )
	return 1;

    slen = strlen(part1)+strlen(part2)+2;
    oldinst = malloc(slen);
    snprintf(oldinst, slen, "%s/%s", part1, part2);

    newinst = afs_config_get_string(remap_cf, "instancemapping", "afs",
				    oldinst, NULL);
    if ( !newinst ) {
	/* turn ONE trailing "/instance" into "/*" */
	char *p;

	if ( (p = strrchr(oldinst, '/')) != NULL) {
	    *++p = '*';
	    *++p = '\0';
	    newinst = afs_config_get_string(remap_cf, "instancemapping",
					    "afs", oldinst, NULL);
	}
    }

    free(oldinst);

    if ( newinst ) {
	if (strcmp(newinst,"\"\"") != 0) {
	    strncpy(inst, newinst, MAXKTCNAMELEN);
	    inst[MAXKTCNAMELEN - 1] = '\0';
	}
	return 0;
    }
    return 1;
}

#if 0
krb5_boolean
rewrite_user(char *name, char *inst, char *realmp, char *new)
{
    krb5_error_code code;
    char *inprinc;
    char *oldinst, *newinst;
    int slen;
    char *newname;

    if (! remap_initialized)
	return FALSE;

    /* if there are any quoted /, @, or \ characters, we won't play */
    if ( strchr(name, '\\') )
	return FALSE;
    if ( inst && strchr(inst, '\\') )
	return FALSE;

    slen = strlen(name)+(inst?strlen(inst):0)+strlen(realmp)+3;
    oldname = malloc(slen);
    snprintf(oldname, slen, "%s%s%s@%s", name, inst?"/":"",
	     inst?inst:"", realmp);

    newname = afs_config_get_string(remap_context,
				     "usermapping",
				     "afs", oldname, NULL);
    free(oldname);
    if (!newname)
	return FALSE;

    if (code == 0)
	new = newname;
    else
	free(newname);

    return (code == 0);
}
#endif
#else
int
init_remap(char *file)
{
    return 1;
}
int
remap_principal(char *part0, char *part1, char *part2, char *realmp,
		char *name, char *inst)
{
    return 1;
}
#endif
